char rcsid_bf_player[] = "$Id: bf_player.c,v 1.3 1992/08/10 17:16:32 pjames Exp $";

#include "functions.h"
#include "list.h"
#include "execute.h"
#include "server.h"
#include "storage.h"
#include "utils.h"

static package
bf_connected_players(Var arglist, Byte next, void *vdata, Objid progr)
{ /* () */
    free_var(arglist);
    return make_var_pack(connected_players());
}


static package
bf_players(Var arglist, Byte next, void *vdata, Objid progr)
{ /* () */
    free_var(arglist);
    return make_var_pack(vardup(players_list));
}

static package
bf_is_player(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object) */
    Var r;
    Objid oid = arglist.v.list[1].v.obj;
    
    free_var(arglist);
    
    if (!retrieve(oid))
	return make_error_pack(E_INVARG);
    
    r.type = TYPE_NUM;
    r.v.num = is_player(oid);
    return make_var_pack(r);
}

static package
bf_notify(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (player, string) */
    
    if (!is_wizard(progr) && progr != arglist.v.list[1].v.obj) {
	free_var(arglist);
	return make_error_pack(E_PERM);
    } 
    
    notify(arglist.v.list[1].v.obj, arglist.v.list[2].v.str);
    free_var(arglist);
    return no_var_pack();
}

static package
bf_idle_seconds(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (player) */
    Var r;
    r.type = TYPE_NUM;
    r.v.num = idle_seconds(arglist.v.list[1].v.obj);
    free_var(arglist);
    if (r.v.num < 0)
	return make_error_pack(E_INVARG);
    else 
	return make_var_pack(r);
}

static package
bf_connected_seconds(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (player) */
    Var r;
    r.type = TYPE_NUM;
    r.v.num = connected_seconds(arglist.v.list[1].v.obj);
    free_var(arglist);
    if (r.v.num < 0)
	return make_error_pack(E_INVARG);
    else 
	return make_var_pack(r);
}

static package
bf_set_player_flag(Var arglist, Byte next, void *vdata, Objid progr) 
{ /* (object, yes/no) */
    Var obj;
    Object *o;
    char bool;
    
    obj = arglist.v.list[1];
    o = retrieve(obj.v.obj);
    bool = is_true(arglist.v.list[2]);
    
    free_var(arglist);
    
    if (!o)
	return make_error_pack(E_INVARG);
    else if (!is_wizard(progr)) 
	return make_error_pack(E_PERM);
    
    if (bool) {
	players_list = setadd(players_list, obj);
	o->flags |= (1 << FLAG_PLAYER);
    } else {
	boot_player(obj.v.obj);
	players_list = setremove(players_list, obj);
	o->flags &= ~(1 << FLAG_PLAYER);
    }
    return no_var_pack();
}

static package
bf_boot_player(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object) */
    Objid oid = arglist.v.list[1].v.obj;
    
    free_var(arglist);
    
    if (oid != progr  &&  !is_wizard(progr)) 
	return make_error_pack(E_PERM);
    
    boot_player(oid);
    return no_var_pack();
}

static package
bf_connection_name(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (player) */
    Objid	who = arglist.v.list[1].v.obj;
    const char *conn_name = player_connection_name(who);
    Var		r;
    
    free_var(arglist);
    if (!is_wizard(progr)  &&  progr != who)
	return make_error_pack(E_PERM);
    else if (!conn_name)
	return make_error_pack(E_INVARG);
    else {
	r.type = TYPE_STR;
	r.v.str = strdup(conn_name);
	return make_var_pack(r);
    }
}

void
register_bf_player(void)
{ 
    (void) register_function("connected_players", 0, 0, bf_connected_players);
    (void) register_function("players", 0, 0, bf_players);
    (void) register_function("is_player", 1, 1, bf_is_player, TYPE_OBJ);
    (void) register_function("idle_seconds", 1, 1, bf_idle_seconds, TYPE_OBJ);
    (void) register_function("connected_seconds", 1, 1, bf_connected_seconds,
			     TYPE_OBJ);
    (void) register_function("set_player_flag", 2, 2, bf_set_player_flag,
			     TYPE_OBJ, TYPE_ANY);
    (void) register_function("boot_player", 1, 1, bf_boot_player, TYPE_OBJ);
    (void) register_function("notify", 2, 2, bf_notify, TYPE_OBJ, TYPE_STR);
    (void) register_function("connection_name", 1, 1, bf_connection_name,
			     TYPE_OBJ);
}

/* $Log: bf_player.c,v $
 * Revision 1.3  1992/08/10  17:16:32  pjames
 * Updated #includes.  Updated to use new registration format.  Built in
 * functions now only receive programmer, instead of entire Parse_Info.
 *
 * Revision 1.2  1992/07/20  23:53:20  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
