char rcsid_bf_server[] = "$Id: bf_server.c,v 1.3 1992/08/10 17:17:09 pjames Exp $";

#include "functions.h"
#include "server.h"
#include "storage.h"
#include "utils.h"
#include "version.h"


static package
bf_server_version(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    r.type = TYPE_STR;
    r.v.str = strdup(server_version);
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_renumber(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    Objid o = arglist.v.list[1].v.obj;
    free_var(arglist);
    
    if (!retrieve(o)) 
	return make_error_pack(E_INVARG);
    else if (!is_wizard(progr)) 
	return make_error_pack(E_PERM);
    
    r.type = TYPE_OBJ;
    r.v.obj = renumber_object(o);
    return make_var_pack(r);
}

static package
bf_reset_max_object(Var arglist, Byte next, void *vdata, Objid progr)
{
    free_var(arglist);
    
    if (!is_wizard(progr))
	return make_error_pack(E_PERM);
    
    reset_db_size();
    return no_var_pack();
}

static package
bf_memory_usage(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    r = memory_usage();
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_shutdown(Var arglist, Byte next, void *vdata, Objid progr)
{
    if (!is_wizard(progr)) { 
	free_var(arglist);
	return make_error_pack(E_PERM);
    } 
    
    request_shutdown(progr, arglist.v.list[1].v.str);
    free_var(arglist);
    return no_var_pack();
}

static package
bf_dump_database(Var arglist, Byte next, void *vdata, Objid progr)
{
    free_var(arglist);
    if (!is_wizard(progr))
	return make_error_pack(E_PERM);
    
    request_checkpoint();
    return no_var_pack();
}

static package
bf_open_network_connection(Var arglist, Byte next, void *vdata, Objid progr)
{
#ifdef OUTBOUND_NETWORK

    Var		r;

    if (!is_wizard(progr))
	return make_error_pack(E_PERM);

    r = open_network_connection(arglist);
    if (r.type == TYPE_ERR)
	return make_error_pack(r.v.err);
    else
	return make_var_pack(r);

#else /* !OUTBOUND_NETWORK */

    /* This function is disabled in this server. */

    free_var(arglist);
    return make_error_pack(E_PERM);

#endif    
}

void
register_bf_server(void)
{ 
    (void) register_function("server_version", 0, 0, bf_server_version);
    (void) register_function("renumber", 1, 1, bf_renumber, TYPE_OBJ);
    (void) register_function("reset_max_object", 0, 0, bf_reset_max_object);
    (void) register_function("memory_usage", 0, 0, bf_memory_usage);
    (void) register_function("shutdown", 1, 1, bf_shutdown, TYPE_STR);
    (void) register_function("dump_database", 0, 0, bf_dump_database);
    (void) register_function("open_network_connection", 0, -1,
		      bf_open_network_connection);
}

/* $Log: bf_server.c,v $
 * Revision 1.3  1992/08/10  17:17:09  pjames
 * Updated #includes.  Updated to use new registration format.  Built in
 * functions now only receive programmer, instead of entire Parse_Info.
 *
 * Revision 1.2  1992/07/20  23:54:00  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
