char rcsid_bf_str_list[] = "$Id: bf_str_list.c,v 1.6 1992/08/13 23:17:54 pavel Exp $";

#include "my-string.h"

#include "bf_register.h"
#include "functions.h"
#include "list.h"
#include "utils.h"
#include "random.h"
#include "regex.h"
#include "storage.h"
#include "streams.h"

/* crypt */
extern const char  *crypt(const char *, const char *); /* BSDism */

/* 1 function common to strings and lists */

static package
bf_length(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    switch(arglist.v.list[1].type) {
      case TYPE_LIST:
	r.type = TYPE_NUM;
	r.v.num = arglist.v.list[1].v.list[0].v.num;
	break;
      case TYPE_STR:
	r.type = TYPE_NUM;
	r.v.num = strlen(arglist.v.list[1].v.str);
	break;
      default:
	free_var(arglist);
	return make_error_pack(E_TYPE);
	break;
    }
    
    free_var(arglist);
    return make_var_pack(r);
}

/* 6 functions specific to lists  */

static package 
bf_setadd(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    
    r = setadd(vardup(arglist.v.list[1]), vardup(arglist.v.list[2]));
    free_var(arglist);
    return make_var_pack(r);
}


static package
bf_setremove(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    
    r = setremove(vardup(arglist.v.list[1]), arglist.v.list[2]);
    free_var(arglist);
    return make_var_pack(r);
}


static package
bf_listappend(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    if (arglist.v.list[0].v.num == 2)
	r = listappend(vardup(arglist.v.list[1]), vardup(arglist.v.list[2]));
    else
	r = listinsert(vardup(arglist.v.list[1]), vardup(arglist.v.list[2]),
		       arglist.v.list[3].v.num + 1);
    free_var(arglist);
    return make_var_pack(r);
}


static package
bf_listinsert(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    if (arglist.v.list[0].v.num == 2)
	r = listinsert(vardup(arglist.v.list[1]), vardup(arglist.v.list[2]), 1);
    else
	r = listinsert(vardup(arglist.v.list[1]), 
		       vardup(arglist.v.list[2]), arglist.v.list[3].v.num);
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_listdelete(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    if (arglist.v.list[2].v.num <= 0 
	|| arglist.v.list[2].v.num > arglist.v.list[1].v.list[0].v.num) {
	free_var(arglist);
	return make_error_pack(E_RANGE);
    } else {
	r = listdelete(vardup(arglist.v.list[1]), arglist.v.list[2].v.num);
    }
    free_var(arglist);
    return make_var_pack(r);
}


static package
bf_listset(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var r;
    if (arglist.v.list[3].v.num <= 0 
	|| arglist.v.list[3].v.num > arglist.v.list[1].v.list[0].v.num) {
	free_var(arglist);
	return make_error_pack(E_RANGE);
    } else {
	r = listset(vardup(arglist.v.list[1]), 
		    vardup(arglist.v.list[2]), arglist.v.list[3].v.num);
    }
    free_var(arglist);
    return make_var_pack(r);
}

/* 5 functions specific to str */

static package
bf_strsub(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (source, what, with [, case-matters]) */
    Var r;
    int	case_matters = 0;
    
    if (arglist.v.list[0].v.num == 4)
	case_matters = arglist.v.list[4].v.num;
    if (arglist.v.list[2].v.str[0] == '\0') {
	free_var(arglist);
	return make_error_pack(E_INVARG);
    } else {
	r.type = TYPE_STR;
	r.v.str = strdup(strsub(arglist.v.list[1].v.str, arglist.v.list[2].v.str,
				arglist.v.list[3].v.str, case_matters));
	
	free_var(arglist);
	return make_var_pack(r);
    }
}

static package
bf_crypt(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (string, [salt]) */
    Var r;
    char	salt[3];
    static	char saltstuff[] =
	"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
    extern const char *crypt(const char *, const char *);
    
    if (arglist.v.list[0].v.num == 1) {
	salt[0] = saltstuff[RANDOM() % strlen(saltstuff)];
	salt[1] = saltstuff[RANDOM() % strlen(saltstuff)];
    } else {
	salt[0] = arglist.v.list[2].v.str[0];
	salt[1] = arglist.v.list[2].v.str[1];
    }
    salt[2] = '\0';
    r.type = TYPE_STR;
    r.v.str = strdup(crypt(arglist.v.list[1].v.str, salt));
    
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_strcmp(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (string1, string2) */
    Var r;
    
    r.type = TYPE_NUM;
    r.v.num = strcmp(arglist.v.list[1].v.str, arglist.v.list[2].v.str);
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_index(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (source, what [, case-matters]) */
    Var r;
    int case_matters = 0;
    
    if (arglist.v.list[0].v.num == 3)
	case_matters = arglist.v.list[3].v.num;
    r.type = TYPE_NUM;
    r.v.num = strindex(arglist.v.list[1].v.str, arglist.v.list[2].v.str, 
		       case_matters);
    
    free_var(arglist);
    return make_var_pack(r);
}

static package
bf_rindex(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (source, what [, case-matters]) */
    Var r;
    
    int case_matters = 0;
    
    if (arglist.v.list[0].v.num == 3)
	case_matters = arglist.v.list[3].v.num;
    r.type = TYPE_NUM;
    r.v.num = strrindex(arglist.v.list[1].v.str, arglist.v.list[2].v.str, 
			case_matters);
    
    free_var(arglist);
    return make_var_pack(r);
}

static struct re_pattern_buffer buf;
static char fastmap[256];
static char trans_table[256];
static struct re_registers regs;

void
init_translate_table()
{
    int i;
    for (i = 0; i < 256; i++)
	trans_table[i] = i;
    for (i = 'a'; i <= 'z'; i++)
	trans_table[i] = i + 'A' - 'a';
}

void
setup_regex()
{
    init_translate_table();
    buf.buffer = 0;
    buf.allocated = 0;
    buf.used = 0;
    buf.fastmap = fastmap;
    buf.translate = 0;
    buf.fastmap_accurate = 0;
}

Var
do_match(Var arglist, int reverse)
{
    char *subject, *pattern;
    int subject_length;
    int match_ans;
    const char *compile_error;
    Var ans;

    subject = arglist.v.list[1].v.str;
    pattern = arglist.v.list[2].v.str;
    subject_length = strlen(subject);
    buf.translate = trans_table;
    if (arglist.v.list[0].v.num == 3) 
	if (is_true(arglist.v.list[3]))
	    buf.translate = 0;
    compile_error = re_compile_pattern(pattern, strlen(pattern), &buf);
    if (compile_error) {
	ans.type = TYPE_ANY;
	return ans;
    }
    re_compile_fastmap(&buf);
    match_ans = re_search(&buf, subject, subject_length,
	     reverse ? subject_length - 1 : 0,
	     reverse ? -subject_length : subject_length, &regs);
    if (match_ans != -1) {
	int loop;
	
	ans = new_list(4);
	ans.v.list[1].type = TYPE_NUM;
	ans.v.list[2].type = TYPE_NUM;
	ans.v.list[4].type = TYPE_STR;
	ans.v.list[1].v.num = regs.start[0]+1;
	ans.v.list[2].v.num = regs.end[0];
	ans.v.list[3] = new_list(RE_NREGS-1);
	ans.v.list[4].v.str = strdup(subject);
	for (loop = 1; loop < RE_NREGS; loop++) {
	    ans.v.list[3].v.list[loop] = new_list(2);
	    ans.v.list[3].v.list[loop].v.list[1].type = TYPE_NUM;
	    ans.v.list[3].v.list[loop].v.list[2].type = TYPE_NUM;
	    if (regs.start[loop] == -1) {
		ans.v.list[3].v.list[loop].v.list[1].v.num = 0;
		ans.v.list[3].v.list[loop].v.list[2].v.num = -1;
	    } else {
		ans.v.list[3].v.list[loop].v.list[1].v.num =
		    regs.start[loop] + 1;
		ans.v.list[3].v.list[loop].v.list[2].v.num =
		    regs.end[loop];
	    }
	}
    } else 
	ans = new_list(0);
    return ans;
}

static package
bf_match(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var ans;
    
    ans = do_match(arglist, 0);
    free_var(arglist);
    if (ans.type == TYPE_ANY)
	return make_error_pack(E_INVARG);
    else
	return make_var_pack(ans);
}

static package
bf_rmatch(Var arglist, Byte next, void *vdata, Objid progr)
{
    Var ans;

    ans = do_match(arglist, 1);
    free_var(arglist);
    if (ans.type == TYPE_ANY)
	return make_error_pack(E_INVARG);
    else
	return make_var_pack(ans);
}

int
invalid_pair(int num1, int num2, int max)
{
    if (num1 == 0 && num2 == -1)
	return 0;
    if (num1 > 0 && num2 >= num1 && num2 <= max)
	return 0;
    return 1;
}

int
check_subs_list(Var subs)
{
    char *subj;
    int subj_length, loop;
    
    if (subs.type != TYPE_LIST || subs.v.list[0].v.num != 4
	|| subs.v.list[1].type != TYPE_NUM
	|| subs.v.list[2].type != TYPE_NUM
	|| subs.v.list[3].type != TYPE_LIST
	|| subs.v.list[3].v.list[0].v.num != RE_NREGS - 1
	|| subs.v.list[4].type != TYPE_STR)
	return 1;
    subj = subs.v.list[4].v.str;
    subj_length = strlen(subj);
    if (invalid_pair(subs.v.list[1].v.num, subs.v.list[2].v.num,
		     subj_length))
	return 1;
    
    for (loop = 1; loop < RE_NREGS; loop++) {
	Var pair;
	pair = subs.v.list[3].v.list[loop];
	if (pair.type != TYPE_LIST 
	    || pair.v.list[0].v.num != 2 
	    || pair.v.list[1].type != TYPE_NUM 
	    || pair.v.list[2].type != TYPE_NUM 
	    || invalid_pair(pair.v.list[1].v.num, pair.v.list[2].v.num,
			     subj_length))
	    return 1;
    }
    return 0;
}

static package
bf_substitute(Var arglist, Byte next, void *vdata, Objid progr)
{
    int template_length, subject_length;
    char *template, *subject;
    Var subs, ans;
    int invarg = 0;
    Stream *s;
    char c = '\0';

    template = arglist.v.list[1].v.str;
    template_length = strlen(template);
    subs = arglist.v.list[2];

    if (check_subs_list(subs)) {
	free_var(arglist);
	return make_error_pack(E_INVARG);
    }
    subject = subs.v.list[4].v.str;
    subject_length = strlen(subject);
    
    s = new_stream(template_length);
    ans.type = TYPE_STR;
    while (c = *(template++)) {
	switch(c) {
	  case '%':
	    {
		Var pair;
		int start = 0, end = 0;
		c = *(template++);
		if (c >= '1' && c <= '9') {
		    pair = subs.v.list[3].v.list[c-'0'];
		    start = pair.v.list[1].v.num-1;
		    end = pair.v.list[2].v.num-1;
		} else if (c == '0') {
		    start = subs.v.list[1].v.num-1;
		    end = subs.v.list[2].v.num-1;
		} else 
		    invarg = 1;
		if (!invarg) {
		    int where;
		    for (where = start; where <= end; where++)
			stream_add_char(s, subject[where]);
		}
		break;
	    }
	  default:
	    stream_add_char(s, c);
	}
	if (invarg)
	    break;
    }
    
    free_var(arglist);
    if (!invarg)
	ans.v.str = strdup(reset_stream(s));
    free_stream(s);
    if (invarg) 
	return make_error_pack(E_INVARG);
    else 
	return make_var_pack(ans);
    
}

/* register */

void
register_bf_str_list(void)
{
    /* list */
    (void) register_function("length", 1, 1, bf_length, TYPE_ANY);
    (void) register_function("setadd", 2, 2, bf_setadd, TYPE_LIST, TYPE_ANY);
    (void) register_function("setremove", 2, 2, bf_setremove,
			     TYPE_LIST, TYPE_ANY);
    (void) register_function("listappend", 2, 3, bf_listappend,
			     TYPE_LIST, TYPE_ANY, TYPE_NUM);
    (void) register_function("listinsert", 2, 3, bf_listinsert,
			     TYPE_LIST, TYPE_ANY, TYPE_NUM);
    (void) register_function("listdelete", 2, 2, bf_listdelete,
			     TYPE_LIST, TYPE_NUM);
    (void) register_function("listset", 3, 3, bf_listset,
			     TYPE_LIST, TYPE_ANY, TYPE_NUM);

    /* string */
    setup_regex();
    (void) register_function("match", 2, 3, bf_match, TYPE_STR, TYPE_STR,
			     TYPE_NUM);
    (void) register_function("rmatch", 2, 3, bf_rmatch, TYPE_STR, TYPE_STR,
			     TYPE_NUM);
    (void) register_function("substitute", 2, 2, bf_substitute, TYPE_STR,
			     TYPE_LIST);
    (void) register_function("crypt", 1, 2, bf_crypt, TYPE_STR, TYPE_STR);
    (void) register_function("index", 2, 3, bf_index,
			     TYPE_STR, TYPE_STR, TYPE_NUM);
    (void) register_function("rindex", 2, 3, bf_rindex,
			     TYPE_STR, TYPE_STR, TYPE_NUM);
    (void) register_function("strcmp", 2, 2, bf_strcmp, TYPE_STR, TYPE_STR);
    (void) register_function("strsub", 3, 4, bf_strsub,
			     TYPE_STR, TYPE_STR, TYPE_STR, TYPE_NUM);
}

/* $Log: bf_str_list.c,v $
 * Revision 1.6  1992/08/13  23:17:54  pavel
 * Added missing #include "storage.h"...
 *
 * Revision 1.5  1992/08/13  22:11:06  pjames
 * Fixed to match fixes in regex.h
 *
 * Revision 1.4  1992/08/13  21:30:58  pjames
 * Added bf_match(), bf_rmatch(), and bf_substitute() for regular
 * expression support.
 *
 * Revision 1.3  1992/08/10  17:28:55  pjames
 * Updated #includes.  Used new registration method.  Built in function
 * receives only programmer, not whole Parse_Info.
 *
 * Revision 1.2  1992/07/20  23:54:17  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
