#ifndef Execute_h
#define Execute_h 1

#include "bytecodes.h"
#include "config.h"
#include "structures.h"
#include "opcode.h"
#include "program.h"

typedef struct {
  Program *prog;
  Var     *rt_env;   /* same length as prog.var_names */
  Var *base_rt_stack;
  Var *top_rt_stack; /* the stack has a fixed size equal to prog.num_var_names;
			top_rt_stack always points to next empty slot;
			there is no need to check bounds! */  
  unsigned pc;
  Byte bi_func_pc;   /* next == 0 means a normal activation, which just
			returns to the previous activation (caller verb).
			next == 1, 2, 3, ... means the returned value should be
			fed to the bi_func (as specified in bi_func_id) 
			together with the next code. */
  Byte bi_func_id;
  void *bi_func_data;
  Var	temp;	     /* VM's temp register */

  /* verb information */
  Objid this;
  Objid player;
  Objid progr;
  Objid vloc;
  char *verb;
  char *verbname;
  int debug;
  
} activation;

typedef activation	activation_stack[MAX_VERB_DEPTH];

extern void 		free_activation(activation a, char data_too);

typedef struct {
    activation_stack		activ_stack;
    unsigned    		top_activ_stack;
    unsigned    		root_activ_vector;
                                /* root_activ_vector == MAIN_VECTOR
 		                   means root activation is main_vector */
    unsigned    		func_id;
} vmstruct;

typedef vmstruct       *vm;

#define alloc_data(size)   mymalloc(size, M_BI_FUNC_DATA)
#define free_data(ptr)     myfree((void *) ptr, M_BI_FUNC_DATA)

/* call_verb will only return E_MAXREC, E_INVIND, E_VERBNF, E_TYPE (for args) 
   or E_NONE.  the vm will only be changed if E_NONE is returned */
extern enum error call_verb(Objid obj, const char *vname, Var args,
			    int do_pass);

extern int 	setup_activ_for_eval(Program *prog);

extern Var 	do_forked_task(Program *prog, Var *rt_env, activation a,
			       unsigned f_id);
extern Var 	do_input_task(Program *prog, Parse_Info *pi);
extern Var	do_server_task(Objid this, const char *verb, Var args,
			       Objid vloc, char *verbname, Program *prog,
			       Objid progr, int debug, Objid player,
			       Objid caller);

extern void	abort_running_task(void);
extern void     print_error_backtrace(const char *, int);

extern Var      resume_from_previous_vm(vm the_vm, char *possible_input);

extern void 	free_rt_env(Var *, unsigned);
extern Var     *new_rt_env(unsigned);

extern void     write_activ(activation a);
extern char 	read_activ(FILE *f, activation *a, int which_vector);
     
#endif

/* $Log: execute.h,v $
 * Revision 1.3  1992/08/10  17:44:28  pjames
 * Moved several functions declarations to eval_env.h and eval_vm.h
 *
 * Revision 1.2  1992/07/27  18:01:45  pjames
 * Changed name of ct_env to var_names
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
