char rcsid_list[] = "$Id: list.c,v 1.3 1992/08/10 16:53:46 pjames Exp $";

#include "my-string.h"
#include "structures.h"
#include "list.h"
#include "storage.h"
#include "utils.h"

Var
new_list(int size)
{
    Var		new;

    new.type = TYPE_LIST;
    new.v.list = (Var *) mymalloc((size + 1) * sizeof (Var), M_LIST);
    new.v.list[0].type = TYPE_NUM;
    new.v.list[0].v.num = size;
    return new;
}

Var
setadd(Var list, Var value)
{
    if (ismember(value, list)) {
	free_var(value);
	return list;
    }
    return listappend(list, value);
}

Var
setremove(Var list, Var value)
{
    int		i;

    if (i = ismember(value, list)) {
	return listdelete(list, i);
    } else {
	return list;
    }
}

int
ismember(Var lhs, Var rhs)
{
    int		i;

    for (i = 1; i <= rhs.v.list[0].v.num; i++) {
	if (equality(lhs, rhs.v.list[i])) {
	    return i;
	}
    }
    return 0;
}

Var
listset(Var list, Var value, int pos)
{
    free_var(list.v.list[pos]);
    list.v.list[pos] = value;
    return list;
}

static Var
doinsert(Var list, Var value, int pos)
{
    Var		new;
    int		i;

    new = new_list(list.v.list[0].v.num + 1);
    for (i = 1; i < pos; i++)
	new.v.list[i] = list.v.list[i];
    new.v.list[pos] = value;
    for (i = pos; i <= list.v.list[0].v.num; i++)
	new.v.list[i + 1] = list.v.list[i];
    myfree((void *) list.v.list, M_LIST);
    return new;
}

Var
listinsert(Var list, Var value, int pos)
{
    if (pos <= 0)
	pos = 1;
    else if (pos > list.v.list[0].v.num)
	pos = list.v.list[0].v.num + 1;
    return doinsert(list, value, pos);
}

Var
listappend(Var list, Var value)
{
    return doinsert(list, value, list.v.list[0].v.num + 1);
}

Var
listdelete(Var list, int pos)
{
    Var		new;
    int		i;

    new = new_list(list.v.list[0].v.num - 1);
    for (i = 1; i < pos; i++) {
	new.v.list[i] = list.v.list[i];
    }
    for (i = pos + 1; i <= list.v.list[0].v.num; i++)
	new.v.list[i - 1] = list.v.list[i];
    free_var(list.v.list[pos]);		/* free deleted value */
    myfree((void *) list.v.list, M_LIST);	/* free old list */
    return new;
}

Var
listconcat(Var first, Var second)
{
    int	lsecond = second.v.list[0].v.num;
    int	lfirst = first.v.list[0].v.num;
    Var	new;
    int i;
    
    new = new_list(lsecond + lfirst);
    for (i = 1; i <= lfirst; i++)
	new.v.list[i] = first.v.list[i];
    for (i = 1; i <= lsecond; i++)
	new.v.list[i + lfirst] = second.v.list[i];
    myfree(second.v.list, M_LIST);
    myfree(first.v.list, M_LIST);

    return new;
}

Var
sublist(Var list, int lower, int upper)
{
    if (lower > upper) {
	free_var(list);
	return new_list(0);
    } else {
	Var		r;
	int		i;
	
	r = new_list(upper - lower + 1);
	for (i = 1; i < lower; i++)
	    free_var(list.v.list[i]);
	for (i = lower; i <= upper; i++)
	    r.v.list[i - lower + 1] = list.v.list[i];
	for (i = upper + 1; i <= list.v.list[0].v.num; i++)
	    free_var(list.v.list[i]);
	myfree((void *) list.v.list, M_LIST);
	return r;
    }
}

Var
substr(Var str, int lower, int upper)
{
    Var		r;

    r.type = TYPE_STR;
    if (lower > upper)
	r.v.str = strdup("");
    else {
	r.v.str = (char *) mymalloc (upper - lower + 2, M_STRING);
	str.v.str[upper] = '\0';
	strcpy(r.v.str, str.v.str + lower - 1);
    }
    free_var(str);
    return r;
}

Var
strget(Var str, Var i)
{
    Var		r;

    r.type = TYPE_STR;
    r.v.str = strdup(" ");
    r.v.str[0] = str.v.str[i.v.num - 1];
    return r;
}

/* $Log: list.c,v $
 * Revision 1.3  1992/08/10  16:53:46  pjames
 * Updated #includes.
 *
 * Revision 1.2  1992/07/21  00:04:04  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
