char rcsid_match[] = "$Id: match.c,v 1.3 1992/08/10 16:54:00 pjames Exp $";

#include "my-stdlib.h"
#include "my-string.h"

#include "structures.h"
#include "match.h"
#include "parse_command.h"
#include "storage.h"
#include "unparse.h"
#include "utils.h"

static Var *
aliases(Objid oid)
{
    static Var	answer[2];
    int		pid;

    if (match_property("aliases", oid, &pid)) {
	Var	value;

	value = retrieve(oid)->propval[pid].var;
	if (value.type == TYPE_LIST)
	    return value.v.list;
	/* simulate an empty list */
	answer[0].type = TYPE_NUM;
	answer[0].v.num = 0;
	return answer;
    } else {
	/* simulate a list of one string */
	answer[0].type = TYPE_NUM;
	answer[0].v.num = 1;
	answer[1].type = TYPE_STR;
	answer[1].v.str = retrieve(oid)->name;
	return answer;
    }
}
    

static Objid
match_contents(Objid player, const char *name)
{
    Objid	loc;
    int		step;
    Objid	oid;
    int		lname = strlen(name);
    Objid	exact = NOTHING, partial = FAILED_MATCH;

    if (!retrieve(player))
	return FAILED_MATCH;
    loc = retrieve(player)->location;

    for (oid = player, step = 0; step < 2; oid = loc, step++) {
	Object *o = retrieve(oid);

	if (!o)
	    continue;
	for (oid = o->contents; oid != NOTHING; oid = retrieve(oid)->next) {
	    Var	       *names = aliases(oid);
	    int		i;

	    for (i = 1; i <= names[0].v.num; i++) {
		if (names[i].type != TYPE_STR)
		    continue;
		if (!mystrncasecmp(names[i].v.str, name, lname)) {
		    if (names[i].v.str[lname] == '\0') { /* exact match */
			if (exact == NOTHING || exact == oid)
			    exact = oid;
			else
			    return AMBIGUOUS;
		    } else {				/* partial match */
			if (partial == FAILED_MATCH || partial == oid)
			    partial = oid;
			else
			    partial = AMBIGUOUS;
		    }
		}
	    }
	}
    }
    if (exact != NOTHING)
	return exact;
    else
	return partial;
}

Objid
match_object(Objid player, const char *name)
{
    if (name[0] == '\0')
	return NOTHING;
    if (name[0] == '#') {
	char   *p;
	Objid	r = strtol(name + 1, &p, 10);

	if (*p != '\0' || !retrieve(r))
	    return FAILED_MATCH;
	return r;
    }
    if (!retrieve(player))
	return FAILED_MATCH;
    if (!mystrcasecmp(name, "me"))
	return player;
    if (!mystrcasecmp(name, "here"))
	return retrieve(player)->location;
    return match_contents(player, name);
}

Verbdef *
match_verbdef(char *vname, Objid oid, int  *vid)
{
    Verbdef    *v;
    int		num;
    char       *p;

    num = strtol(vname, &p, 10);
    if (*p != '\0')
	num = -1;

    *vid = 0;
    v = retrieve(oid)->verbdefs;
    for (; v; v = v->next)  {
	if (verbcasecmp(v->name, vname) || *vid == num)
	    return v;
	(*vid)++;
    }
    *vid = FAILED_MATCH;
    return 0;
}

Propdef *
match_property(const char *pname, Objid oid, Propid *pid)
{
    Object	*c;
    Propdef	*p;
    
    if (match_pseudo_property(pname, pid)) {
	return (Propdef *) 1;
    }

    *pid = 0;
    for (c = retrieve(oid); c; c = retrieve(c->parent)) {
	for (p = c->propdefs; p; p = p->next) {
	    if (!mystrcasecmp(p->name, pname))
		return p;
	    (*pid)++;
	}
    }
    
    *pid = FAILED_MATCH;
    return 0;
}

Verbdef *
match_verbref(Objid player, char *vref, Objid *what, int *vid)
{
    char       *w = strchr(vref, ':');
    char       *vname;
    Objid	oid;

    if (w == 0  ||  w[1] == '\0') {
	*what = 0;
	*vid = NOTHING;
	return 0;
    }
    *w = '\0';
    vname = w + 1;
    if (vref[0] == '$')
	oid = get_system_object(vref + 1);
    else
	oid = match_object(player, vref);
    if (retrieve(oid)) {
	*what = oid;
	return match_verbdef(vname, oid, vid);
    }
    *what = FAILED_MATCH;
    *vid = 0;
    return 0;
}

enum vopt
match_option(const char *opt)
{
    if (!mystrcasecmp(opt, "none")) return OPT_NONE;
    if (!mystrcasecmp(opt, "any")) return OPT_ANY;
    if (!mystrcasecmp(opt, "this")) return OPT_THIS;
    return OPT_FAIL;
}

/* $Log: match.c,v $
 * Revision 1.3  1992/08/10  16:54:00  pjames
 * Updated #includes.
 *
 * Revision 1.2  1992/07/21  00:04:40  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
