/* server.h */

/*
 * This describes the complete set of procedures that a network implementation
 * is expected to use from the rest of the server.  See 'network.h' for the
 * complete set of procedures that a network implementation must provide.
 */

#ifndef Server_H
#define Server_H 1

#include "network.h"

typedef struct {
    void       *ptr;
} server_handle;

extern server_handle	server_new_connection(network_handle h);
				/* Called by the network whenever a new
				 * player connection is made.  By this call,
				 * the network and server exchange tokens
				 * representing the connection for use in
				 * later calls on each other.
				 */

extern void		server_receive_line(server_handle h, const char *line);
				/* The given line has been received as input
				 * on the specified connection.  'line' does
				 * not end in a newline; any such bytes have
				 * been removed by the network.  The characters
				 * in 'line' are all either spaces or non-
				 * whitespace ASCII characters.
				 */

extern void		server_close(server_handle h);
				/* The specified connection has been broken
				 * for some reason not in the server's control.
				 * Effective immediately, the network will no
				 * longer use the given server_handle and the
				 * server should not use the corresponding
				 * network_handle.
				 */

#endif /* Server_H */


/*
 * The following procedures should not be called by a network implementation;
 * they are exported from the server module to other parts of the program.
 */

#include "structures.h"

extern void	player_connected(Objid old_id, Objid new_id,
				 int is_newly_created);
extern const char *player_connection_name(Objid player);
extern void	notify(Objid player, const char *message);
extern Var	connected_players(void);
extern int	idle_seconds(Objid player);
extern int	connected_seconds(Objid player);
extern void	boot_player(Objid player);
extern void	panic(const char *message);
extern void	request_shutdown(Objid player, const char *reason);
extern void	request_checkpoint(void);
extern Var	open_network_connection(Var arglist);

/* $Log: server.h,v $
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
