char rcsid_sym_table[] = "$Id: sym_table.c,v 1.4 1992/08/10 16:50:28 pjames Exp $";

#include "my-stdio.h"

#include "ast.h"
#include "log.h"
#include "server.h"
#include "storage.h"
#include "structures.h"
#include "sym_table.h"
#include "utils.h"

Names *
new_names(unsigned max_size)
{
    Names      *names = (Names *) mymalloc(sizeof(Names), M_NAMES);
    names->names = (char **) mymalloc(sizeof(char *) * max_size, M_NAMES);

    names->max_size = max_size;
    names->size = 0;

    return names;
}

Names *
new_builtin_names(void)
{
    static Names       *builtins = 0;

    if (builtins == 0) {
	builtins = new_names(FIRST_USER_SLOT);
	builtins->size = FIRST_USER_SLOT;

	builtins->names[SLOT_NUM] = strdup("NUM");
	builtins->names[SLOT_OBJ] = strdup("OBJ");
	builtins->names[SLOT_STR] = strdup("STR");
	builtins->names[SLOT_LIST] = strdup("LIST");
	builtins->names[SLOT_ERR] = strdup("ERR");
	builtins->names[SLOT_PLAYER] = strdup("player");
	builtins->names[SLOT_THIS] = strdup("this");
	builtins->names[SLOT_CALLER] = strdup("caller");
	builtins->names[SLOT_VERB] = strdup("verb");
	builtins->names[SLOT_ARGS] = strdup("args");
	builtins->names[SLOT_ARGSTR] = strdup("argstr");
	builtins->names[SLOT_DOBJ] = strdup("dobj");
	builtins->names[SLOT_DOBJSTR] = strdup("dobjstr");
/*  taken out to match code_elt.c
	builtins->names[SLOT_PREP] = strdup("prep");
*/
	builtins->names[SLOT_PREPSTR] = strdup("prepstr");
	builtins->names[SLOT_IOBJ] = strdup("iobj");
	builtins->names[SLOT_IOBJSTR] = strdup("iobjstr");
    }
    return copy_names(builtins);
}

Names *
copy_names(Names *old)
{
    Names      *new = new_names(old->size);
    unsigned	i;

    new->size = old->size;
    for (i = 0; i < new->size; i++)
	new->names[i] = strdup(old->names[i]);

    return new;
}

int
find_name(Names *names, const char *str)
{
    unsigned	i;

    for (i = 0; i < names->size; i++)
	if (!mystrcasecmp(names->names[i], str))
	    return i;
    return -1;
}

unsigned
find_or_add_name(Names **names, char *str)
{
  unsigned i;

  for (i = 0; i < (*names)->size; i++)
    if (!mystrcasecmp((*names)->names[i], str)) {/* old name */
      return i;
    }
  
  if ((*names)->size == (*names)->max_size) {
    unsigned	old_max = (*names)->max_size;
    Names      *new = new_names(old_max * 2);
    unsigned	i;

    for (i = 0; i < old_max; i++)
      new->names[i] = (*names)->names[i];
    new->size = old_max;
    myfree((void *) (*names)->names, M_NAMES);
    myfree((void *) *names, M_NAMES);
    *names = new;
  }
  (*names)->names[(*names)->size] = strdup(str);
  return (*names)->size++;
}

void
free_names(Names *names)
{
    unsigned	i;

    for (i = 0; i < names->size; i++)
	myfree((void *) names->names[i], M_STRING);
    myfree((void *) names->names, M_NAMES);
    myfree((void *) names, M_NAMES);
}

Env *
new_env(unsigned size)
{
    Env	       *new = (Env *) mymalloc(sizeof(Env) +
				       (size - 1) * sizeof(Var),
				       M_ENV);
    unsigned	i;

    new->size = size;
    for (i = 0; i < size; i++)
	new->vars[i].type = TYPE_ANY;

    return new;
}

Env *
copy_env(Env *old)
{
    Env	       *new = new_env(old->size);
    unsigned	i;

    for (i = 0; i < old->size; i++)
	new->vars[i] = vardup(old->vars[i]);

    return new;
}

void
free_env(Env *env)
{
    unsigned	i;

    for (i = 0; i < env->size; i++)
	free_var(env->vars[i]);
    myfree((void *) env, M_ENV);
}

Var
env_ref(Env *env, unsigned slot)
{
    if (slot < env->size)
	return vardup(env->vars[slot]);
    else {
	Var	v;
	
	panic("ENV_REF: Slot index out of bounds.");
	v.type = TYPE_NUM;
	v.v.num = 0;
	return v;
    }
}

void
env_set(Env *env, unsigned slot, Var v)
{
    if (slot < env->size) {
	free_var(env->vars[slot]);
	env->vars[slot] = vardup(v);
    } else
	panic("ENV_SET: Slot index out of bounds.");
}    

/*
void
write_env(FILE *f, Names *names, Env *env)
{
    unsigned	i;

    if (names->size != env->size)
	panic("WRITE_ENV: Sizes don't match!");

    fprintf(f, "%d variables\n", env->size);
    for (i = 0; i < env->size; i++) {
	write_string(f, names->names[i]);
	write_var(f, env->vars[i]);
    }
}
*/
int
read_env(FILE *f, Names **names, Env **env)
{
    int		count;
    unsigned	i;

    if (fscanf(f, "%d variables\n", &count) != 1) {
	errlog("READ_ENV: Bad count.\n");
	return -1;
    }

    *names = new_names(count);
    (*names)->size = count;
    *env = new_env(count);

    for (i = 0; i < count; i++) {
	(*names)->names[i] = strdup(read_string(f));
	(*env)->vars[i] = read_var(f);
    }
    return 0;
}


/* $Log: sym_table.c,v $
 * Revision 1.4  1992/08/10  16:50:28  pjames
 * Updated #includes.
 *
 * Revision 1.3  1992/07/27  18:23:48  pjames
 * Changed M_CT_ENV to M_NAMES.  Freed some memory that wasn't being
 * freed when growing or freeing a list of names.
 *
 * Revision 1.2  1992/07/21  00:07:06  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
