#ifndef Tasks_H
#define Tasks_H 1

#include "structures.h"
#include "execute.h" /* vm */

typedef struct {
    void       *ptr;
} task_queue;

typedef enum { TASK_INPUT, TASK_FORKED, TASK_SUSPENDED } task_kind;

extern void	set_task_kind(task_kind kind);

extern task_queue	new_task_queue(Objid player);
extern void		free_task_queue(task_queue q);

extern void		enqueue_input_task(task_queue, const char *);
extern int		enqueue_forked_task(Program *program, activation a,
					    Var *rt_env, unsigned f_index, 
					    unsigned after_seconds);
extern void 		enqueue_suspended_task(vm the_vm,
					       unsigned after_seconds);
extern enum error	make_reading_task(vm the_vm, Var opt_player);

extern int		next_task_start(void);
extern void		run_ready_tasks(void);
extern void		run_server_task(Objid player, Objid what,
					const char *verb, Var args,
					Var *result);

extern int		task_id(void);
extern Var		queued_tasks(Objid owner);
extern enum error	kill_task(int id, Objid owner);

extern int		output_delimiters(Objid player, const char **prefix,
					  const char **suffix);

extern void		write_task_queue(FILE *);
extern int		read_task_queue(FILE *);

#endif /* !Tasks_H */

/* $Log: tasks.h,v $
 * Revision 1.2  1992/08/10  16:48:00  pjames
 * Changed Parse_Info to activation in enqueue_forked_task argument list,
 * because all important information is now stored in activation.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 * */
